<?php
// tòmò web analytics
// Copyright since 2011 by Yves Goergen
// Website: http://unclassified.de/go/tomo
// Licence: GNU General Public License (GPL) Version 3
// Encoding: UTF-8 (without BOM) (auto-detect: °°°°°)
//
// This file is derived from:
// UNB Components
// Copyright since 2007 by Yves Goergen
// Website: http://unclassified.de/go/unb2
//
// This file contains the UnbDebug class.

// UNB debugging functions.
//
class UnbDebug
{
	// Specifies whether errors shall be printed out with a full call trace.
	private static $showTrace = false;
	
	// Specifies whether trace messages are written to a log file.
	private static $traceToFile = false;
	
	// Specifies the maximum length of stack trace argument value dumps.
	private static $maxArgLength = 40;

	// Random per-request tag to distinguish concurrent requests in log files.
	private static $requestTag;

	// Private constructor, does nothing but preventing an unwanted instantiation of this class.
	//
	private function __construct()
	{
	}
	
	// Returns a random per-request tag to distinguish concurrent requests in log files.
	//
	public static function GetRequestTag()
	{
		if (!self::$requestTag)
		{
			self::$requestTag = mt_rand(10000, 99999);
		}
		return self::$requestTag;
	}

	// Writes a text message into the trace file.
	//
	// The message is written to the file unb.trace in the same directory as the UNB class library,
	// together with a time stamp at the beginning of the line. Each message is appended at the end
	// of the file.
	//
	// text = (string) Single-line text message
	//
	public static function Trace($text)
	{
		if (!self::$traceToFile) return;
		
		#$text = date('Y-m-d H:i:s ') . $text . "\n";
		list($usec, $time) = explode(' ', microtime());
		$text = date('Y-m-d H:i:s', $time) . '.' . substr($usec, 2, 6) .
			' [' . self::GetRequestTag() . '] ' .
			$text . "\n";
		file_put_contents(dirname(__FILE__) . '/tomo.' . date('Y-m-d', $time) . '.trace', $text, FILE_APPEND);
	}
	
	// Returns a variable dump instead of echo'ing it.
	//
	public static function Dump($var)
	{
		ob_start();
		var_dump($var);
		return trim(ob_get_clean());
	}

	// Trims path levels outside of this webspace directory from a file name.
	//
	// This makes a file name shorter, more relevant and doesn't reveal the absolute path on the
	// web server should it be displayed to the public.
	//
	// fileName = (string) Rooted file name to be trimmed. Relative (non-rooted) file names will not work!
	//
	// Returns: (string) File name relative to this webspace's document root.
	//
	public static function TrimFilename($fileName)
	{
		$fileName = str_replace('\\', '/', $fileName);
		$root = rtrim(str_replace('\\', '/', $_SERVER['DOCUMENT_ROOT']), '/');
		if (substr($fileName, 0, strlen($root)) == $root)
			return substr($fileName, strlen($root));
		return $fileName;
	}

	// Formats the function call arguments of a call trace element.
	//
	private static function FormatArgs($args)
	{
		$str = '';
		$c = 0;
		if (is_array($args)) foreach ($args as $a)
		{
			if ($c++) $str .= ', ';
			if (is_string($a) && mb_strlen($a) > self::$maxArgLength)
				$a = mb_substr($a, 0, self::$maxArgLength - 1) . '…';
			if (is_string($a))
				$str .= "'" . str_replace(array('\\', "\n", "\r", "\t"), array('\\\\', '\n', '\r', '\t'), $a) . "'";
			else if (is_bool($a))
				$str .= $a ? 'true' : 'false';
			else if ($a === null)
				$str .= 'null';
			else if (is_object($a))
				$str .= get_class($a);
			else
				$str .= $a;
		}
		return $str;
	}

	// Formats a call trace.
	//
	// This method is used for automatic error and exception handlers. It may also be used for
	// other debugging purposes, but in those cases it should be called with an integer as only
	// parameter or no parameters at all.
	//
	// fileName = (int) Number of levels to skip in the call trace, 0 if not specified
	//            (string) File name as reported in the error event
	//            (Exception) Read all data from the Exception object
	// line = (int) Line number as reported in the error event (ignored if the first parameter is no string)
	//
	// Returns: (string) Formatted call trace with class, function and file names and line numbers.
	//
	public static function GetCallTrace($fileName = null, $line = null)
	{
		$skip = 2;
		$start = 0;

		if (is_object($fileName) && $fileName instanceof Exception)
		{
			$ex = $fileName;
			$line = $ex->getLine();
			$fileName = $ex->getFile();
		}
		else if (is_int($fileName))
		{
			$skip = $fileName;
			$start = 1;
		}
		else if (!isset($fileName))
		{
			$skip = 0;
			$start = 1;
		}

		$str = '';
		if ($ex)
			$trace = $ex->getTrace();
		else
			$trace = array_slice(debug_backtrace(), $skip);
		for ($i = $start; $i <= count($trace); $i++)
		{
			if ($i < count($trace))
				$str .= '  in ' . $trace[$i]['class'] . $trace[$i]['type'] . $trace[$i]['function'] .
					'(' . self::FormatArgs($trace[$i]['args']) . ')';
			else
				$str .= '  in main()';
			if ($i == 0)
				$str .= ' at ' . htmlspecialchars(self::TrimFilename($fileName)) . ':' . $line . "\n";
			else
				$str .= ' at ' . htmlspecialchars(self::TrimFilename($trace[$i - 1]['file'])) . ':' . $trace[$i - 1]['line'] . "\n";
		}
		return rtrim($str);
	}

	// Custom PHP error handler.
	//
	// This function can be registered as PHP error handler to print information about an unhandled
	// error in a PHP script. See http://php.net/set_error_handler for details.
	//
	// See StartErrorHandling
	//
	public static function ErrorHandler($errno, $errstr, $errfile, $errline, $errcontext)
	{
		$stop = false;
		switch ($errno)
		{
			case E_WARNING: $type = 'PHP warning'; break;
			case E_NOTICE: $type = 'PHP notice'; break;
			case E_USER_ERROR: $type = 'User-defined error'; $stop = true; break;
			case E_USER_WARNING: $type = 'User-defined warning'; break;
			case E_USER_NOTICE: $type = 'User-defined notice'; break;
			case E_STRICT: $type = 'PHP strict notice'; break;
			case E_RECOVERABLE_ERROR: $type = 'PHP recoverable error'; $stop = true; break;
			case E_DEPRECATED: $type = 'PHP deprecated notice'; break;
			case E_USER_DEPRECATED: $type = 'User-defined deprecated notice'; break;
			default: $type = 'Unknown error'; $stop = true; break;
		}
		echo '<p class="' . ($stop ? 'errorbox' : 'warningbox') . '" style="white-space: pre-wrap;"><strong>' . $type . '</strong>: ';
		if (in_array(ini_get('html_errors'), array(1, '1', 'on', 'yes', 'true')))
			echo $errstr;
		else
			echo htmlspecialchars($errstr);
		if (self::$showTrace)
		{
			echo '<br />';
			echo str_replace("\n", '<br />', self::GetCallTrace($errfile, $errline));   // No \n after <br>! We're in pre-wrap.
			#echo 'Context: <span style="white-space: pre;">' . str_replace("\n\n", "\n", htmlspecialchars(print_r($errcontext, true))) . '</span>';
		}
		echo '</p>';
		if ($stop) die();
	}

	// Custom PHP exception handler.
	//
	// This function can be registered as PHP exception handler to print information about an
	// unhandled exception in a PHP script. See http://php.net/set_exception_handler for details.
	//
	// See StartErrorHandling
	//
	public static function ExceptionHandler($ex)
	{
		echo '<p class="errorbox" style="white-space: pre-wrap;"><strong>Unhandled ' . get_class($ex) . '</strong>: ' . htmlspecialchars($ex->getMessage());
		if ($ex->getCode())
			echo ' (Code ' . $ex->getCode() . ')';
		if (self::$showTrace)
		{
			echo '<br />';
			echo str_replace("\n", '<br />', self::GetCallTrace($ex));   // No \n after <br>! We're in pre-wrap.
		}
		echo '</p>';
	}

	// Registers custom error handlers for unhandled PHP errors and exceptions.
	//
	// This method should be called after the HTML page has been started and is ready for further
	// page elements, as the error handlers will print out formatted HTML code.
	//
	// showTrace = (bool) Specifies whether errors shall be printed out with a full call trace.
	//
	public static function StartErrorHandling($showTrace = false)
	{
		self::$showTrace = $showTrace;

		ini_set('html_errors', 1);
		ini_set('docref_root', 'http://php.net/');
		set_error_handler(array('UnbDebug', 'ErrorHandler'), error_reporting());
		set_exception_handler(array('UnbDebug', 'ExceptionHandler'));
	}

} // class UnbDebug

?>
